//
//  PLSConnection.m
//  Pulse
//
//  Created by Matt Attaway on 2/1/14.
//  Copyright (c) 2014 Zen of the Monkey. All rights reserved.
//


#import "P4Connection.h"
#import "P4Message.h"


@implementation P4Connection


- (id) init
{
    self = [super init];
    
    if(self)
    {
        self.p4path = @"/usr/local/bin/p4";
    }
    
    return self;
}


- (id) initWithSettings:(NSString*)port user:(NSString*)user client:(NSString*)client path:(NSString*)path charset:(NSString*)charset
{
    self = [super init];
    
    if(self)
    {
        self.path = path;
        self.port = port;
        self.user = user;
        self.client = client;
        self.p4path  = @"/usr/local/bin/p4";
        self.charset = charset;
    }
    
    return self;
}


- (P4Message*)revertPath:(NSString*)path options:(NSUInteger)opts
{
    NSArray* paths = [[NSArray alloc] initWithObjects:path, nil];
    return [self revertPaths:paths options:opts];
}


- (P4Message*)revertPaths:(NSArray *)paths options:(NSUInteger)opts
{
    // bail if no path has been provided. it makes Perforce sad
    if([paths count] == 0)
    {
        return [[P4Message alloc] initWithError:@"Did not run command. Need a path for revert"];
    }
    
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSMutableArray* args = [[NSMutableArray alloc] initWithArray: @[@"-p", self.port, @"-u", self.user, @"-c",
                                                                    self.client, @"-d", self.path, @"-C", self.charset,
                                                                    @"revert"]];
    // handle the command options
    if(opts & P4ConnectionRevertServerOnlyOption)
    {
        [args addObject:@"-k"];
    }

    task.arguments  = [args arrayByAddingObjectsFromArray:paths];
    
    NSPipe* pipe = [NSPipe pipe];
    [task setStandardOutput: pipe];
    
    NSFileHandle* tfile = [pipe fileHandleForReading];
    
    [task launch];
    [task waitUntilExit];
    
    NSData* data = [tfile readDataToEndOfFile];
    
    P4Message* results = [[P4Message alloc] init];
    results.message = [[NSString alloc] initWithData: data encoding: NSUTF8StringEncoding];
    
    return results;
}

- (P4Message*)submit:(NSString*)description
{
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSString* command = @"submit";
    task.arguments  = @[@"-p", self.port, @"-u", self.user, @"-c", self.client, @"-d", self.path, @"-C", self.charset, command, @"-d", description];
    
    NSPipe* pipe = [NSPipe pipe];
    [task setStandardOutput: pipe];
    
    NSFileHandle* tfile = [pipe fileHandleForReading];
    
    [task launch];
    [task waitUntilExit];
    
    NSData* data = [tfile readDataToEndOfFile];
    
    P4Message* results = [[P4Message alloc] init];
    results.message = [[NSString alloc] initWithData: data encoding: NSUTF8StringEncoding];
    
    return results;
}


- (P4Message*) syncFiles:(NSString *)path
{
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSString* command = @"sync";
    task.arguments  = @[@"-p", self.port, @"-u", self.user, @"-c", self.client, @"-d", self.path, @"-C", self.charset, command, path];
    
    NSPipe* out = [NSPipe pipe];
    [task setStandardOutput: out];
    NSFileHandle* ofile = [out fileHandleForReading];
    
   // NSPipe* err = [NSPipe pipe];
   // [task setStandardError: err];
   // NSFileHandle* efile = [err fileHandleForReading];

    [task launch];
    [task waitUntilExit];
    
    NSData* odata = [ofile readDataToEndOfFile];
    //NSData* edata = [efile readDataToEndOfFile];
    
    P4Message* results = [[P4Message alloc] init];
    results.message = [[NSString alloc] initWithData: odata encoding: NSUTF8StringEncoding];
    
    return results;
}


- (P4Message*) reconcilePath:(NSString *)path options:(NSUInteger)opts
{
    NSArray* paths = [[NSArray alloc] initWithObjects:path, nil];
    return [self reconcilePaths:paths options:opts];
}


- (P4Message*) reconcilePaths:(NSArray *)paths options:(NSUInteger)opts
{
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    // XXX: hardcoded .p4ignore for now
    NSDictionary* env = [[NSProcessInfo processInfo] environment];
    [env setValue:@".p4ignore" forKey:@"P4IGNORE"];
    [task setEnvironment:env];
    
    NSMutableArray* args = [[NSMutableArray alloc] initWithArray: @[@"-p", self.port, @"-u", self.user, @"-c",
                                                                    self.client, @"-d", self.path, @"-C", self.charset,
                                                                    @"reconcile", @"-f"]];
    // handle the command options
    if(opts & P4ConnectionReconcileAddOption)
    {
        [args addObject:@"-a"];
    }

    if(opts & P4ConnectionReconcileDeleteOption)
    {
        [args addObject:@"-d"];
    }
    
    if(opts & P4ConnectionReconcileEditOption)
    {
        [args addObject:@"-e"];
    }
    
    task.arguments  = [args arrayByAddingObjectsFromArray:paths];
    
    NSPipe* out = [NSPipe pipe];
    [task setStandardOutput: out];
    NSFileHandle* ofile = [out fileHandleForReading];
    
    NSPipe* err = [NSPipe pipe];
    [task setStandardError: err];
    NSFileHandle* efile = [err fileHandleForReading];
    
    [task launch];
    [task waitUntilExit];
    
    NSData* odata = [ofile readDataToEndOfFile];
    NSData* edata = [efile readDataToEndOfFile];
    
    P4Message* results = [[P4Message alloc] init];
    results.message = [[NSString alloc] initWithData: odata encoding: NSUTF8StringEncoding];
    results.error   = [[NSString alloc] initWithData: edata encoding: NSUTF8StringEncoding];
    
    return results;
}


- (P4Message*)getLatestChangeInClient
{
    P4Message* results = [[P4Message alloc] init];
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSDictionary* env = [[NSProcessInfo processInfo] environment];
    [env setValue:@".p4ignore" forKey:@"P4IGNORE"];
    [task setEnvironment:env];
    
    NSString* clientPath = [NSString stringWithFormat:@"//%@/...", self.client ];
    
    NSString* command = @"changes";
    task.arguments  = @[@"-ztag", @"-p", self.port, @"-u", self.user, @"-c", self.client, @"-d", self.path, @"-C", self.charset, command, @"-m1", clientPath ];
    
    NSPipe* out = [NSPipe pipe];
    [task setStandardOutput: out];
    NSFileHandle* ofile = [out fileHandleForReading];
    
    NSPipe* err = [NSPipe pipe];
    [task setStandardError: err];
    NSFileHandle* efile = [err fileHandleForReading];
    
    [task launch];
    
    int count = 0;
    while([task isRunning])
    {
        [NSThread sleepForTimeInterval:1.0f];
        
        count++;
        if(count > 10)
        {
            NSLog(@"I GOT TIRED OF WAITING");
            results.error = @"Operation timed out for reasons I don't fully understand";
            return results;
        }
    }
    
    NSData* odata = [ofile readDataToEndOfFile];
    NSData* edata = [efile readDataToEndOfFile];
    
    NSString* string = [[NSString alloc] initWithData: odata encoding: NSUTF8StringEncoding];
    
    NSArray* lines = [string componentsSeparatedByString:@"\n"];
    
    results.error   = [[NSString alloc] initWithData: edata encoding: NSUTF8StringEncoding];
    
    for( NSString* line in lines)
    {
        NSArray* words = [line componentsSeparatedByString:@" "];
        
        if ([words count] > 1)
        {
            NSString* value = [[words subarrayWithRange:NSMakeRange(2, [words count] - 2)] componentsJoinedByString:@" "];
            [results.results setObject:value forKey:[words objectAtIndex:1]];
        }
    }
    
    return results;
}


- (P4Message*)getChanges:(NSInteger)high low:(NSInteger)low
{
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSString* clientPath = [NSString stringWithFormat:@"//%@/...@%li,%li", self.client, low, high ];
    
    NSString* command = @"changes";
    task.arguments  = @[@"-ztag", @"-p", self.port, @"-u", self.user, @"-c", self.client, @"-d", self.path, @"-C", self.charset, command, clientPath ];
    
    NSPipe* out = [NSPipe pipe];
    [task setStandardOutput: out];
    NSFileHandle* ofile = [out fileHandleForReading];
    
    NSPipe* err = [NSPipe pipe];
    [task setStandardError: err];
    NSFileHandle* efile = [err fileHandleForReading];
    
    [task launch];
    [task waitUntilExit];
    
    NSData* odata = [ofile readDataToEndOfFile];
    NSData* edata = [efile readDataToEndOfFile];
    
    NSString* string = [[NSString alloc] initWithData: odata encoding: NSUTF8StringEncoding];

    NSArray* lines = [string componentsSeparatedByString:@"\n"];
    NSMutableDictionary* change = nil;
    P4Message* results = [[P4Message alloc] init];
    
    results.error   = [[NSString alloc] initWithData: edata encoding: NSUTF8StringEncoding];
    
    for( NSString* line in lines)
    {
        NSArray* words = [line componentsSeparatedByString:@" "];

        if ([words count] > 2 && [[words objectAtIndex:0] isEqualToString:@"..."])
        {
            // create our dictionary if needs be
            if ([[words objectAtIndex:1]  isEqual: @"change"])
            {
                if(change)
                {
                    [results.list addObject:change];
                }
                
                change = [[NSMutableDictionary alloc] init];
            }
            
            NSString* value = [[words subarrayWithRange:NSMakeRange(2, [words count] - 2)] componentsJoinedByString:@" "];
            [change setObject:value forKey:[words objectAtIndex:1]];
        }
    }
    
    if(change)
    {
        [results.list addObject:change];
    }

    return results;
}


- (P4Message*)login:(NSString*)password
{
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSString* command = @"login";
    task.arguments  = @[@"-p", self.port, @"-u", self.user, @"-c", self.client, @"-d", self.path, @"-C", self.charset, command];
    
    NSPipe* out = [NSPipe pipe];
    [task setStandardOutput: out];
    NSFileHandle* ofile = [out fileHandleForReading];
    
    NSPipe* err = [NSPipe pipe];
    [task setStandardError: err];
    NSFileHandle* efile = [err fileHandleForReading];
    
    NSPipe* in = [NSPipe pipe];
    [task setStandardInput: in];
    NSFileHandle* ifile = [in fileHandleForWriting];
    
    [task launch];
    
    [ifile writeData: [password dataUsingEncoding: NSUTF8StringEncoding]];
    [ifile closeFile];
    
    [task waitUntilExit];
    
    NSData* odata = [ofile readDataToEndOfFile];
    NSData* edata = [efile readDataToEndOfFile];

    P4Message* results = [[P4Message alloc] init];
    results.error   = [[NSString alloc] initWithData: edata encoding: NSUTF8StringEncoding];
    results.message = [[NSString alloc] initWithData: odata encoding: NSUTF8StringEncoding];
    
    return results;
}


- (BOOL)isConnected
{
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSString* command = @"help";
    task.arguments  = @[@"-ztag", @"-p", self.port, @"-u", self.user, @"-c", self.client, @"-d", self.path, @"-C", self.charset, command];
        
    NSPipe* err = [NSPipe pipe];
    [task setStandardError: err];
    NSFileHandle* efile = [err fileHandleForReading];
    
    [task launch];
    [task waitUntilExit];
    
    NSData* edata = [efile readDataToEndOfFile];

    P4Message* results = [[P4Message alloc] init];
    results.error   = [[NSString alloc] initWithData: edata encoding: NSUTF8StringEncoding];
    
    return ![results isConnectionError];
}


- (BOOL)isLoggedIn
{
    NSTask* task = [[NSTask alloc] init];
    task.launchPath = self.p4path;
    task.currentDirectoryPath = self.path;
    
    NSString* command = @"login";
    task.arguments  = @[@"-ztag", @"-p", self.port, @"-u", self.user, @"-c", self.client, @"-d", self.path, @"-C", self.charset, command, @"-s"];
    
    NSPipe* err = [NSPipe pipe];
    [task setStandardError: err];
    NSFileHandle* efile = [err fileHandleForReading];
    
    [task launch];
    [task waitUntilExit];
    
    NSData* edata = [efile readDataToEndOfFile];
    
    P4Message* results = [[P4Message alloc] init];
    results.error   = [[NSString alloc] initWithData: edata encoding: NSUTF8StringEncoding];
    
    return ![results isLoginError];
}

@end







