import java.util.Date;
import java.util.List;

import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.ChangelistStatus;
import com.perforce.p4java.core.IChangelist;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.FileSpecOpStatus;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.exception.RequestException;
import com.perforce.p4java.impl.generic.core.Changelist;
import com.perforce.p4java.impl.mapbased.server.Server;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4java.server.ServerFactory;
import com.perforce.p4java.option.client.EditFilesOptions;

public class SimpleTestApp {

	public static void main(String[] args) {
		try {		
			IOptionsServer server = ServerFactory.getOptionsServer("p4java://localhost:1666", null);
			
			server.setUserName("matt");
			server.setCharsetName("utf16");
			server.connect();
			IClient client = server.getClient("mac-matt");

			server.setCurrentClient(client);


			Changelist changeListImpl = new Changelist(
											IChangelist.UNKNOWN,
											client.getName(),
											server.getUserName(),
											ChangelistStatus.NEW,
											new Date(),
											"Changelist demo new changelist",
											false,
											(Server) server
					);
			IChangelist changelist = client.createChangelist(changeListImpl);

			EditFilesOptions editFilesOptions = new EditFilesOptions().setChangelistId(changelist.getId());
			
			List<IFileSpec> editList = client.editFiles(FileSpecBuilder.makeFileSpecList("//depot/SampleText.txt"), null);

			changelist.update();
			List<IFileSpec> submitFiles = changelist.submit(false);
			if (submitFiles != null) {
				for (IFileSpec fileSpec : submitFiles) {
					if (fileSpec != null) {
						if (fileSpec.getOpStatus() == FileSpecOpStatus.VALID) {
							System.out.println("submitted: " + fileSpec.getDepotPathString());
						} else if (fileSpec.getOpStatus() == FileSpecOpStatus.INFO){
							System.out.println(fileSpec.getStatusMessage());
						} else if (fileSpec.getOpStatus() == FileSpecOpStatus.ERROR){
							System.err.println(fileSpec.getStatusMessage());
						}
					}
				}
			}	
		} catch (RequestException rexc) {
			System.err.println(rexc.getDisplayString());
			rexc.printStackTrace();
		} catch (Exception exc) {
			System.err.println(exc.getLocalizedMessage());
			exc.printStackTrace();
		}
	}	
}

